//-----------------------------------------------------------------------------
//
//  FILE: eeprom.c
//
//  DESCRIPTION:
//      This file contains the eCos driver routines for the EEPROM attached
//      to I2C bus.
//
//  FUNCTIONS DEFINED:
//
//      EEPROM_Init         initialize the EEPROM interface
//      EEPROM_Read         read the EEPROM
//      EEPROM_Write        write the EEPROM
//
//  MODIFICATION HISTORY:
//
//

#include <pkgconf/system.h>
#include <cyg/hal/drv_api.h>
#include <cyg/io/i2c.h>
#include <cyg/io/eeprom.h>
#include <string.h>
#include <errno.h>

#ifdef CYGPKG_KERNEL
#include <cyg/kernel/kapi.h>            // cyg_thread_delay()
#else
#include <cyg/hal/hal_diag.h>           // HAL_DELAY_US()
#endif


#define EEPROM_DEV_ADDR     (0xa0)
#define EEPROM_DEV_SIZE     (0x80)
#define EEPROM_PAGE_SIZE    (8)

static cyg_drv_mutex_t EEPROM_mutex;

#ifndef min
#define min(x, y)       (((x) < (y)) ? (x) : (y))
#endif

//-----------------------------------------------------------------------------
//
//  Initialize the EEPROM interface.
//

int
EEPROM_Init(void)
{
    cyg_drv_mutex_init(&EEPROM_mutex);
    return I2C_Init(I2C_FREQ(100000));
}


//-----------------------------------------------------------------------------
//
//  Read the EEPROM over the I2C bus.
//

int
EEPROM_Read(cyg_uint32 addr, void* pBuf, size_t bufSize)
{
    cyg_uint8 eepromAddr = addr;
    I2C_Msg_t msgs[2];
    int status;

    // serialize access to the device
    if (!cyg_drv_mutex_lock(&EEPROM_mutex))
    {
        return -EINTR;
    }

    // write message to set the EEPROM address
    msgs[0].devAddr = EEPROM_DEV_ADDR | I2C_WR;
    msgs[0].pBuf = &eepromAddr;
    msgs[0].bufSize = sizeof(eepromAddr);

    // read message
    msgs[1].devAddr = EEPROM_DEV_ADDR | I2C_RD;
    msgs[1].pBuf = pBuf;
    msgs[1].bufSize = bufSize;

    // transfer the messages
    status = I2C_Transfer(2, msgs);

    cyg_drv_mutex_unlock(&EEPROM_mutex);
    return status;
}


//-----------------------------------------------------------------------------
//
//  Write the EEPROM over the I2C bus.
//

int
EEPROM_Write(cyg_uint32 addr, void const* pBuf, size_t bufSize)
{
    cyg_uint8 addr_page[1 + EEPROM_PAGE_SIZE];
    cyg_uint8 const* pBufByte = (cyg_uint8 const*)pBuf;
    I2C_Msg_t msg;

    // serialize access to the device
    if (!cyg_drv_mutex_lock(&EEPROM_mutex))
    {
        return -EINTR;
    }

    msg.devAddr = EEPROM_DEV_ADDR | I2C_WR;
    msg.pBuf = addr_page;

    while (bufSize > 0)
    {
        size_t nBytes;
        int status;

        // write at most a page at a time
        nBytes = min(bufSize, EEPROM_PAGE_SIZE);

        // don't cross a page boundary
        if (addr%EEPROM_PAGE_SIZE)
        {
            nBytes = min(nBytes, EEPROM_PAGE_SIZE - addr%EEPROM_PAGE_SIZE);
        }

        // build the write message
        addr_page[0] = addr;
        memcpy(&addr_page[1], pBufByte, nBytes);
        msg.bufSize = nBytes + 1;
        addr += nBytes;
        pBufByte += nBytes;
        bufSize -= nBytes;

        // transfer the message
        status = I2C_Transfer(1, &msg);
        if (status != I2C_STATUS_SUCCESS)
        {
            cyg_drv_mutex_unlock(&EEPROM_mutex);
            return status;
        }

        // delay 10 msec
#ifdef CYGPKG_KERNEL
        {
            static cyg_resolution_t const res = CYGNUM_KERNEL_COUNTERS_RTC_RESOLUTION;
            cyg_thread_delay(10*1000*1000ULL*res.divisor/res.dividend);
        }
#else
        HAL_DELAY_US(10*1000);
#endif
    }

    cyg_drv_mutex_unlock(&EEPROM_mutex);
    return ENOERR;
}
