//-----------------------------------------------------------------------------
//
//  FILE: i2c.c
//
//  DESCRIPTION:
//      This file contains the eCos device driver routines for the Samsung
//      S3C4510x I2C bus controller.
//
//  FUNCTIONS DEFINED:
//
//      I2C_Init            initialize the bus controller
//      I2C_Transfer        do the I2C message transfers
//
//  MODIFICATION HISTORY:
//
//

#include <cyg/hal/hal_io.h>
#include <cyg/hal/drv_api.h>
#include <cyg/io/i2c.h>


static cyg_drv_mutex_t I2C_mutex;


//-----------------------------------------------------------------------------
//
//  Initialize the I2C bus controller.
//

int
I2C_Init(cyg_uint32 prescale)
{
    // reset the bus controller
    HAL_WRITE_UINT32(KS32C_I2CCON, KS32C_I2C_CON_RESET);

    // set the bus frequency
    HAL_WRITE_UINT32(KS32C_I2CPS, prescale);

    cyg_drv_mutex_init(&I2C_mutex);
    return I2C_STATUS_SUCCESS;
}


//-----------------------------------------------------------------------------
//
//  Transfer the I2C messages.
//

int
I2C_Transfer(size_t nMsg, I2C_Msg_t* pMsgs)
{
    cyg_uint32 i2cCon;

    // serialize access to the I2C bus
    if (!cyg_drv_mutex_lock(&I2C_mutex))
    {
        return I2C_STATUS_MUTEX;
    }

    // is the bus free ?
    HAL_READ_UINT32(KS32C_I2CCON, i2cCon);
    if (i2cCon & KS32C_I2C_CON_BUSY)
    {
        cyg_drv_mutex_unlock(&I2C_mutex);
        return I2C_STATUS_BUSY;
    }

    // transfer the messages
    for (; nMsg > 0; --nMsg, ++pMsgs)
    {
        // generate the start condition
        HAL_WRITE_UINT32(KS32C_I2CCON, KS32C_I2C_CON_START);

        // send the device address
        HAL_WRITE_UINT32(KS32C_I2CBUF, pMsgs->devAddr);
        do
        {
            HAL_READ_UINT32(KS32C_I2CCON, i2cCon);
        } while ((i2cCon & KS32C_I2C_CON_BF) == 0);

        // check if the slave ACK'ed the device address
        if (i2cCon & KS32C_I2C_CON_LRB)
        {
            // generate the stop condition
            HAL_WRITE_UINT32(KS32C_I2CCON, KS32C_I2C_CON_STOP);
            cyg_drv_mutex_unlock(&I2C_mutex);
            return I2C_STATUS_ADDR_NAK;
        }

        // read the message ?
        if (pMsgs->devAddr & I2C_RD)
        {
            cyg_uint8* pBuf = pMsgs->pBuf;
            size_t bufSize = pMsgs->bufSize;
            cyg_uint32 i2cBuf;

            // read more than one byte ?
            if (--bufSize > 0)
            {
                // enable ACK
                HAL_WRITE_UINT32(KS32C_I2CCON, KS32C_I2C_CON_ACK);

                while (bufSize-- > 0)
                {
                    do
                    {
                        HAL_READ_UINT32(KS32C_I2CCON, i2cCon);
                    } while ((i2cCon & KS32C_I2C_CON_BF) == 0);

                    // read the data byte
                    HAL_READ_UINT32(KS32C_I2CBUF, i2cBuf);
                    *pBuf++ = i2cBuf;
                }
            }

            // disable ACK
            HAL_WRITE_UINT32(KS32C_I2CCON, 0);
            do
            {
                HAL_READ_UINT32(KS32C_I2CCON, i2cCon);
            } while ((i2cCon & KS32C_I2C_CON_BF) == 0);

            // read the data byte
            HAL_READ_UINT32(KS32C_I2CBUF, i2cBuf);
            *pBuf++ = i2cBuf;
        }

        // write the message
        else
        {
            size_t i;

            for (i = 0; i < pMsgs->bufSize; ++i)
            {
                HAL_WRITE_UINT32(KS32C_I2CBUF, pMsgs->pBuf[i]);
                do
                {
                    HAL_READ_UINT32(KS32C_I2CCON, i2cCon);
                } while ((i2cCon & KS32C_I2C_CON_BF) == 0);

                // check if the slave ACK'ed the data byte
                if (i2cCon & KS32C_I2C_CON_LRB)
                {
                    // generate the stop condition
                    HAL_WRITE_UINT32(KS32C_I2CCON, KS32C_I2C_CON_STOP);
                    cyg_drv_mutex_unlock(&I2C_mutex);
                    return I2C_STATUS_DATA_NAK;
                }
            }
        }

        // generate a restart condition ?
        if (nMsg > 1)
        {
            HAL_WRITE_UINT32(KS32C_I2CCON, KS32C_I2C_CON_RESTART);
        }
    }

    // generate the stop condition
    HAL_WRITE_UINT32(KS32C_I2CCON, KS32C_I2C_CON_STOP);

    cyg_drv_mutex_unlock(&I2C_mutex);
    return I2C_STATUS_SUCCESS;
}
